package com.jair.config;

import com.alibaba.fastjson.JSONObject;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.service.IAccountService;
import com.jair.util.BeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;

@Component
@Slf4j
public class RequestFilter implements Filter {
    @Autowired
    private IAccountService accountService;

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        HttpServletRequest httpServletRequest = (HttpServletRequest) request;
        String requestUri = ((HttpServletRequest) request).getRequestURI();
        if (StringUtils.isEmpty(requestUri)) {
            chain.doFilter(request, response);
            return;
        }
        if (requestUri.contains("/account/login")) {
            chain.doFilter(request, response);
            return;
        }
        TokenInfo tokenInfo = accountService.getTokenInfo(httpServletRequest);
        if (tokenInfo == null) {
            //重定向到登录页
            tokenError(response);
            return;
        }
        httpServletRequest.getServletContext().setAttribute("TokenInfo", tokenInfo);

        chain.doFilter(request, response);
    }

    private void tokenError(ServletResponse response) throws IOException {
        //303状态码表示由于请求对应的资源存在着另一个URI，应使用GET方法定向获取请求的资源
        JSONObject resultVo = BeanUtil.transform(ResultVo.loginTimeOutError("token失效"), JSONObject.class);
        response.setCharacterEncoding("UTF-8");
        response.getWriter().write(resultVo.toJSONString());
    }
}
