package com.jair.entity.common;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 分页页面信息
 *
 * @author caiyt
 */
@Data
@ApiModel(value = "分页页面信息", description = "分页页面的数据封装类")
public class PageInfo<T> {
    @ApiModelProperty(value = "当前页数")
    private Long current;

    @ApiModelProperty(value = "页面数据量")
    private Long size;

    @ApiModelProperty(value = "数据总量")
    private Long total;

    @ApiModelProperty(value = "数据内容")
    private T records;

    @Override
    public String toString() {
        return "PageInfo{current=" + current + ", size=" + size + ", total=" + total + ", records=" + records + '}';
    }

    public PageInfo() {

    }

    public PageInfo(Long total, T records) {
        this.total = total;
        this.records = records;

    }

    public PageInfo(Long current, Long size) {
        this.current = current;
        this.size = size;
    }

    /**
     * 对分页请求参数进行校验更正
     *
     * @param pageInfo
     */
    public static PageInfo checkParam(PageInfo pageInfo) {
        if (pageInfo == null) {
            pageInfo = new PageInfo<>(1L, 10L);
        } else {
            if (pageInfo.getCurrent() == null || pageInfo.getCurrent() <= 0) {
                pageInfo.setCurrent(1L);
            }
            if (pageInfo.getSize() == null || pageInfo.getSize() == 0) {
                pageInfo.setSize(10L);
            }
        }
        return pageInfo;
    }
}