package com.jair.entity.common;


import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 响应封装类
 *
 * @param <T>
 */
@Data
@ApiModel("响应类")
public class ResultVo<T> {
    @ApiModelProperty("响应码：1-成功 2-登陆超时 3-业务异常 4-系统异常")
    private int code;

    @ApiModelProperty("响应消息")
    private String msg;

    @ApiModelProperty("响应体")
    private T data;

    public static <T> ResultVo success() {
        return success(null);
    }

    public static <T> ResultVo success(T data) {
        return resultVo(ResultConstants.SUCCESS,"成功",data);
    }

    public static <T> ResultVo businessError(String message){
        return resultVo(ResultConstants.ILLEGAL_BUSINESS,message,null);
    }

    public static <T> ResultVo accountFrozenError(String message){
        return resultVo(ResultConstants.FROZEN_ACCOUNT,message,null);
    }

    public static <T> ResultVo systemError(String message){
        return resultVo(ResultConstants.SYSTEM_ERROR,message,null);
    }

    public static <T> ResultVo loginTimeOutError(String message){
        return resultVo(ResultConstants.LOGIN_TIMEOUT,message,null);
    }

    private static <T> ResultVo<T> resultVo(int code, String msg, T data){
        ResultVo resultVo = new ResultVo();
        resultVo.setCode(code);
        resultVo.setMsg(msg);
        if (data != null) {
            resultVo.setData(data);
        }
        return resultVo;

    }

    public ResultVo() {
        this(ResultConstants.SUCCESS, "成功");
    }

    public ResultVo(int code, String msg) {
        this(code, msg, null);
    }

    public ResultVo(int code, String msg, T data) {
        this.code = code;
        this.msg = msg;
        this.data = data;
    }
    public boolean isSuccess(){
       return ResultConstants.SUCCESS==this.code;
    }
}