package com.jair.entity.po;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import org.springframework.util.StringUtils;

import java.util.Date;

@Data
public class CompanyPo extends Validation {
    private Integer id;
    /** 公司名称 */
    private String name;

    /** 简称 */
    private String abbr;

    /** 地址 */
    private String address;

    /** 联系人 */
    private String contactName;

    /** 联系电话 */
    private String contactPhone;

    /** 省 */
    private String province;

    /** 市 */
    private String city;

    /** 区 */
    private String district;

    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date expiredDate;

    private Integer status;

    private Integer level;

    @Override
    public ValidateResult validate() {
        if (StringUtils.isEmpty(this.getName())) {
            return ValidateResult.fail("企业名称不能为空");
        }
        if (this.getName().length() < 4) {
            return ValidateResult.fail("企业名称不能少于4位");
        }
        if (StringUtils.isEmpty(this.getAbbr())) {
            return ValidateResult.fail("企业简称不能为空");
        }
        if (this.getAbbr().length() < 2) {
            return ValidateResult.fail("企业简称不能少于2位");
        }
        if (StringUtils.isEmpty(this.getContactName()) || StringUtils.isEmpty(this.getContactPhone())) {
            return ValidateResult.fail("企业联系人和联系电话不能为空");
        }
        return ValidateResult.succeed(this);
    }
}
