package com.jair.entity.po;

import com.jair.entity.constant.AccountTypeEnum;
import lombok.Data;
import org.springframework.util.StringUtils;

@Data
public class CreateAccountPo extends Validation {
    /** 帐号 */
    private String username;

    /** 密码 */
    private String password;

    /** 姓名 */
    private String realName;

    /** 电话 */
    private String phone;

    /** 地址 */
    private String address;

    /** 帐号类型：0超管，1主帐号，2子帐号 */
    private Integer type;

    private Integer companyId;

    @Override
    public ValidateResult validate() {
        if (this.getCompanyId() == null || this.getCompanyId() <= 0) {
            return ValidateResult.fail("未知的企业");
        }
        if (StringUtils.isEmpty(this.getUsername()) || this.getUsername().trim().length() < 4) {
            return ValidateResult.fail("帐号长度不能小于4位");
        }
        if (StringUtils.isEmpty(this.getPassword()) || this.getPassword().length() < 6) {
            return ValidateResult.fail("密码长度不能小于6位");
        }
        if (StringUtils.isEmpty(this.getRealName()) || this.getRealName().trim().length() < 2) {
            return ValidateResult.fail("姓名长度不能小于2位");
        }
        if (StringUtils.isEmpty(this.getPhone()) || this.getPhone().trim().length() != 11) {
            return ValidateResult.fail("电话输入不正确");
        }
        if (this.getType() != AccountTypeEnum.MAIN.getKey() && this.getType() != AccountTypeEnum.SUB.getKey()) {
            return ValidateResult.fail("帐号类型不正确");
        }
        return ValidateResult.succeed(this);
    }
}
