package com.jair.entity.po;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import org.springframework.util.StringUtils;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Data
public class DriverPo extends Validation {

    private Integer id;

    /** 姓名 */
    private String name;

    /** 身份证号码 */
    private String identityNumber;

    /** 性别 */
    private String gender;

    /** 电话 */
    private String phone;

    /** 现住址 */
    private String address;

    /** 紧急联系人 */
    private String contactName;

    /** 紧急联系人电话 */
    private String contactPhone;

    /** 是否有网约车执照：0无，1有 */
    private Integer serviceLicense;

    // 身份证信息
    /** 民族 */
    private String nation;

    /** 出生日期 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date birthday;

    /** 身份证地址 */
    private String originAddress;

    /** 发证机关 */
    private String issueOrganization;

    /** 身份证 有效期-起 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date identityValidPeriodStart;

    /** 身份证 有效期-止 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date identityValidPeriodEnd;

    /** 身份证 正面照片 */
    private Integer frontFileId;

    /** 身份证 反面照片 */
    private Integer backFileId;

    // 驾照信息
    /** 驾驶证号 */
    private String credentialNo;

    /** 档案编号 */
    private String fileNo;

    /** 首次发证日期 */
    private Date firstIssue;

    /** 驾照 准驾车型 */
    private String drivingClass;

    /** 驾照 有效期-起 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date licenseValidPeriodStart;

    /** 驾照 有效期-止 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date licenseValidPeriodEnd;

    /** 驾照 主页照片 */
    private Integer mainFileId;

    /** 驾照 副页照片 */
    private Integer secondFileId;

    @Override
    public ValidateResult validate() {
        if (StringUtils.isEmpty(this.getName()) || this.getName().trim().length() < 2) {
            return ValidateResult.fail("姓名不正确");
        }
        if (StringUtils.isEmpty(this.getGender())) {
            return ValidateResult.fail("性别不正确");
        }
        List<String> genders = Arrays.asList("男", "女");
        if (!genders.contains(this.getGender().trim())) {
            return ValidateResult.fail("性别不正确");
        }
        if (StringUtils.isEmpty(this.getIdentityNumber())) {
            return ValidateResult.fail("身份证号码不正确");
        }
        if (!this.getIdentityNumber().matches("^[1-9]\\d{7}((0\\d)|(1[0-2]))(([0|1|2]\\d)|3[0-1])\\d{3}$|^[1-9]\\d{5}[1-9]\\d{3}((0\\d)|(1[0-2]))(([0|1|2]\\d)|3[0-1])\\d{3}([0-9]|X)$")) {
            return ValidateResult.fail("身份证号码不正确");
        }
        if (StringUtils.isEmpty(this.getPhone()) || this.getPhone().trim().length() != 11) {
            return ValidateResult.fail("手机号不正确");
        }
//        if (StringUtils.isEmpty(this.getAddress())) {
//            return ValidateResult.fail("住址不正确");
//        }
        if (StringUtils.isEmpty(this.getNation())) {
            return ValidateResult.fail("民族不正确");
        }
        if (this.getBirthday() == null) {
            return ValidateResult.fail("出生日期不能为空");
        }
        if (StringUtils.isEmpty(this.getOriginAddress())) {
            return ValidateResult.fail("身份证地址不正确");
        }
        if (StringUtils.isEmpty(this.getIssueOrganization())) {
            return ValidateResult.fail("发证机关不正确");
        }
        if (this.getIdentityValidPeriodStart() == null || this.getIdentityValidPeriodEnd() == null) {
            return ValidateResult.fail("身份证有效期不正确");
        }
        if (this.getIdentityValidPeriodStart().getTime() - this.getIdentityValidPeriodEnd().getTime() >= 0) {
            return ValidateResult.fail("身份证有效期不正确");
        }
        if (this.getFrontFileId() == null || this.getFrontFileId() <= 0) {
            return ValidateResult.fail("缺少身份证正面照片");
        }
        if (this.getBackFileId() == null || this.getBackFileId() <= 0) {
            return ValidateResult.fail("缺少身份证反面照片");
        }

        if (StringUtils.isEmpty(this.getCredentialNo())) {
            return ValidateResult.fail("驾驶证号码不正确");
        }
//        if (StringUtils.isEmpty(this.getFileNo())) {
//            return ValidateResult.fail("驾驶证档案编号不正确");
//        }
        if (this.getFirstIssue() == null) {
            return ValidateResult.fail("首次发证日期不正确");
        }
        if (this.getLicenseValidPeriodStart() == null || this.getLicenseValidPeriodEnd() == null) {
            return ValidateResult.fail("驾驶证有效期不正确");
        }
        if (this.getLicenseValidPeriodStart().getTime() - this.getLicenseValidPeriodEnd().getTime() >= 0) {
            return ValidateResult.fail("驾驶证有效期不正确");
        }
        if (this.getMainFileId() == null || this.getMainFileId() <= 0) {
            return ValidateResult.fail("缺少驾驶证主页照片");
        }
        if (this.getSecondFileId() == null || this.getSecondFileId() <= 0) {
            return ValidateResult.fail("缺少驾驶证副页照片");
        }

        return ValidateResult.succeed(this);
    }
}
