package com.jair.entity.po;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.jair.entity.bo.CfgFee;
import com.jair.entity.constant.ContractTypeEnum;
import lombok.Data;

import java.util.Date;
import java.util.List;

@Data
public class SignContractPo extends Validation {
    private Integer type = 0;

    private Integer vehicleId = 0;
    private Integer sourceId = 0;
    private Integer driverId = 0;

    /** 里程数 */
    private Integer odograph = 0;

    /** ev里程数 */
    private Integer odographEv = 0;

    /** 租赁开始日期 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date startDate;

    /** 租赁结束日期 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date endDate;

    /** 租赁方式 */
    private Integer leaseType;

    /** 租赁周期，xx周，xx月 */
    private Integer leasePeriod;

    /** 应付押金 */
    private Integer payableDepositAmount;
    /** 已付押金 */
    private Integer paidDepositAmount;

    /** 周期租金 */
    private Integer rentAmount = 0;
    /** 第一次已付租金 */
    private Integer firstPaidAmount = 0;

    private List<Integer> files;

    private String remark;

    private List<CfgFeePo> fees;

    @Override
    public ValidateResult validate() {
        if (!this.getType().equals(ContractTypeEnum.SOURCE_CONTRACT.getKey()) && !this.getType().equals(ContractTypeEnum.DRIVER_CONTRACT.getKey())) {
            return ValidateResult.fail("合同类型不正确");
        }
        if (this.getOdograph() == null || this.getOdograph() < 0) {
            return ValidateResult.fail("请输入正确的里程数");
        }
        if (this.getStartDate() == null) {
            return ValidateResult.fail("请选择合同开始日期");
        }
//        if (this.getStartDate().getTime() - this.getEndDate().getTime() >= 0) {
//            return ValidateResult.fail("合约日期选择不正确");
//        }
//        if (this.getEndDate().getTime() - new Date().getTime() <= 0) {
//            return ValidateResult.fail("合约结束日期选择不正确");
//        }
        if (this.getLeaseType() == null) {
            return ValidateResult.fail("请输入正确的出租类型");
        }
        if (this.getLeasePeriod() == null || this.getLeasePeriod() <= 0) {
            return ValidateResult.fail("请输入正确的出租周期");
        }
        if (this.getPayableDepositAmount() == null || this.getPayableDepositAmount() < 0) {
            return ValidateResult.fail("请输入正确的应付押金金额");
        }
        if (this.getPaidDepositAmount() == null || this.getPaidDepositAmount() < 0) {
            return ValidateResult.fail("请输入正确的已付押金金额");
        }
        if (this.getRentAmount() == null || this.getRentAmount() <= 0) {
            return ValidateResult.fail("请输入正确周期租金");
        }
        if (this.getFirstPaidAmount() == null || this.getFirstPaidAmount() <= 0) {
            return ValidateResult.fail("请输入正确首付租金");
        }
        return ValidateResult.succeed(this);
    }
}
