package com.jair.entity.po;

import lombok.Data;

import java.util.Date;
import java.util.List;

@Data
public class UpdateContractPo extends Validation {
    private Integer contractId;

    /** 里程数 */
    private Integer odograph;

    /** ev里程数 */
    private Integer odographEv;

    /** 租赁开始日期 */
    private Date startDate;

    /** 租赁结束日期 */
    private Date endDate;

    /** 租赁方式 */
    private Integer leaseType;

    /** 押金状态：@DepositStatusEnum */
    private Integer depositStatus;

    /** 应付押金 */
    private Integer payableDepositAmount;
    /** 已付押金 */
    private Integer paidDepositAmount;

    /** 返还押金金额 */
    private Integer returnDepositAmount;

    /** 月租金 */
    private Integer rentAmount;

    private String remark;

    private List<Integer> files;

    @Override
    public ValidateResult validate() {
        if (this.getOdograph() < 0) {
            return ValidateResult.fail("请输入正确的里程数");
        }
        if (this.getStartDate() == null || this.getEndDate() == null) {
            return ValidateResult.fail("请选择合约日期");
        }
        if (this.getStartDate().getTime() - this.getEndDate().getTime() >= 0) {
            return ValidateResult.fail("合约日期选择不正确");
        }
        if (this.getEndDate().getTime() - new Date().getTime() <= 0) {
            return ValidateResult.fail("合约结束日期选择不正确");
        }
        if (this.getPayableDepositAmount() == null || this.getPayableDepositAmount() < 0) {
            return ValidateResult.fail("请输入正确的应付押金金额");
        }
        if (this.getPaidDepositAmount() == null || this.getPaidDepositAmount() < 0) {
            return ValidateResult.fail("请输入正确的已付押金金额");
        }
        if (this.getRentAmount() == null || this.getRentAmount() <= 0) {
            return ValidateResult.fail("请输入正确月租金");
        }
        return ValidateResult.succeed(this);
    }
}
