package com.jair.entity.po;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import org.springframework.util.StringUtils;

import java.util.Date;

@Data
public class UpdateVehiclePo extends Validation {
    private Integer id;

    /** 品牌 */
    private String brandName;

    /** 型号 */
    private String model;

    /** 排量 */
    private String displacement;

    /** 颜色 */
    private String color;

    /** 环保标准 */
    private String environmentalStandards;

    /** 动力类型 */
    private String engineType;

    /** 车牌号码 */
    private String plateNo;

    /** 车辆类型：小型轿车 */
    private String type;

    /** 所有人 */
    private String owner;

    /** 住址 */
    private String address;

    /** 使用性质：非营运 */
    private String useCharacter;

    /** 型号 */
    private String licenseModel;

    /** 识别代号 */
    private String vin;

    /** 发动机号码 */
    private String engineNo;

    /** 注册日期 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date registerDate;

    /** 发证日期 */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date issueDate;

    /** 核载人数 */
    private String approvedPerson;

    /** 总质量 */
    private String weight;

    /** 尺寸 */
    private String outlineSize;

    /** 行驶证主页照片 */
    private Integer mainFileId;

    /** 行驶证副页照片 */
    private Integer secondFileId;

    private String remark;

    @Override
    public ValidateResult validate() {
        if (this.getId() == null || this.getId() <= 0) {
            return ValidateResult.fail("车辆id不正确");
        }
        if (StringUtils.isEmpty(this.getPlateNo())) {
            return ValidateResult.fail("请输入完整车牌号");
        }
        this.setPlateNo(this.getPlateNo().toUpperCase());
        if (!this.getPlateNo().matches("^(([京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领][A-Z](([0-9]{5}[DF])|([DF]([A-HJ-NP-Z0-9])[0-9]{4})))|([京津沪渝冀豫云辽黑湘皖鲁新苏浙赣鄂桂甘晋蒙陕吉闽贵粤青藏川宁琼使领][A-Z][A-HJ-NP-Z0-9]{4}[A-HJ-NP-Z0-9挂学警港澳使领]))$")) {
            return ValidateResult.fail("车牌号格式不正确");
        }
        return ValidateResult.succeed(this);
    }
}
