package com.jair.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.jair.entity.bo.Contract;
import com.jair.entity.common.PageInfo;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.po.*;

import java.util.List;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
public interface IContractService extends IService<Contract> {

    /** 合同列表 */
    ResultVo listContracts(TokenInfo tokenInfo, PageInfo<SearchContractPo> pageInfo);

    /** 获取过期合同日志信息 */
    ResultVo getContractLogs(TokenInfo tokenInfo, Integer contractId);

    /** 新签合同 */
    ResultVo signContract(TokenInfo tokenInfo, SignContractPo signContractPo) throws Exception;

    /** 修改合同 */
    ResultVo updateContract(TokenInfo tokenInfo, UpdateContractPo updateContractPo);

    /** 返还押金 */
    ResultVo returnDeposit(TokenInfo tokenInfo, ReturnDepositPo returnDepositPo);

    /** 中止合同 */
    ResultVo terminateContract(TokenInfo tokenInfo, TerminateContractPo terminatePo);

    /** 添加交车信息 */
    ResultVo returnVehicle(TokenInfo tokenInfo, ReturnVehiclePo returnVehiclePo);


    /** 检查车辆是否在存生效中的合同 */
    boolean checkInEffectContract(Integer vehicleId, Integer type);

    /** 获取合同信息 */
    ResultVo getContractById(TokenInfo tokenInfo, Integer contractId);

    Contract getContractById(Integer companyId, Integer contractId);

    /** 获取合同信息 */
    Contract getContractByType(Integer companyId, Integer type, Integer id);

    String genContractCode(Integer type);

    /** 获取过期合同，修改合同状态为0 */
    void checkEndedContracts();

    /** 统计接口 */
    /** 根据创建时间获取合同数量 */
    Integer getContractCountByCreateTime(TokenInfo tokenInfo, String createTime);

    /** 获取全部合同数量 */
    Integer getAllContractCount(TokenInfo tokenInfo);

    /** 获取执行中合同数量 */
    Integer getValidContractCount(TokenInfo tokenInfo);

    /** 获取签约中司机数量 */
    Integer getSignedDriverCount(TokenInfo tokenInfo);

    /** 获取签约中车辆数量 */
    Integer getSignedVehicleCount(TokenInfo tokenInfo);
}
