package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.Company;
import com.jair.entity.common.PageInfo;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.AccountTypeEnum;
import com.jair.entity.po.CompanyPo;
import com.jair.entity.po.SearchCompanyPo;
import com.jair.entity.po.ValidateResult;
import com.jair.entity.vo.CompanyVo;
import com.jair.entity.vo.DashboardVo;
import com.jair.mapper.CompanyMapper;
import com.jair.service.*;
import com.jair.util.BeanUtil;
import com.jair.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 企业主体 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
@Slf4j
public class AnalyticsServiceImpl implements IAnalyticsService {

    @Autowired
    private IAccountService accountService;
    @Autowired
    private ICompanyService companyService;
    @Autowired
    private IDriverService driverService;
    @Autowired
    private IVehicleService vehicleService;
    @Autowired
    private IContractService contractService;
    @Autowired
    private IPlanningService planningService;

    @Override
    public ResultVo dashboard(TokenInfo tokenInfo) throws ParseException {
        DashboardVo dashboardVo = new DashboardVo();
        String currentMonthDate = DateUtil.format2Date(new Date(), "yyyy-MM");
        String lastMonthDate = DateUtil.format2Date(DateUtil.getAddTime(new Date(), Calendar.MONTH, -1), "yyyy-MM");
        String todayDate = DateUtil.format2Date(new Date());
        String yesterdayDate = DateUtil.format2Date(DateUtil.getAddDayTime(-1));

        Integer currentMonthPlanAmount = planningService.getPlanAmountByPlanDate(tokenInfo, currentMonthDate);
        Integer currentMonthActualAmount = planningService.getActualAmountByCreateTime(tokenInfo, currentMonthDate);
        Integer todayPlanAmount = planningService.getPlanAmountByPlanDate(tokenInfo, todayDate);
        Integer todayActualAmount = planningService.getActualAmountByCreateTime(tokenInfo, todayDate);
        // 本月应收
        dashboardVo.setCurrentMonthPlanAmount(currentMonthPlanAmount == null ? 0 : currentMonthPlanAmount);
        // 本月已收
        dashboardVo.setCurrentMonthActualAmount(currentMonthActualAmount == null ? 0 : currentMonthActualAmount);
        // 今日应收
        dashboardVo.setTodayPlanAmount(todayPlanAmount == null ? 0 : todayPlanAmount);
        // 今日已收
        dashboardVo.setTodayActualAmount(todayActualAmount == null ? 0 : todayActualAmount);

        // 全部司机数量
        dashboardVo.setAllDriverCount(driverService.getAllDriverCount(tokenInfo));
        // 签约中司机数量
        dashboardVo.setSignedDriverCount(contractService.getSignedDriverCount(tokenInfo));

        // 全部车辆数量
        dashboardVo.setAllVehicleCount(vehicleService.getAllVehicleCount(tokenInfo));
        // 签约中车辆数量
        dashboardVo.setSignedVehicleCount(contractService.getSignedVehicleCount(tokenInfo));

        // 合同相关统计
        dashboardVo.setAllContract(contractService.getAllContractCount(tokenInfo));
        dashboardVo.setValidContract(contractService.getValidContractCount(tokenInfo));

        dashboardVo.setCurrentMonthContract(contractService.getContractCountByCreateTime(tokenInfo, currentMonthDate));
        dashboardVo.setLastMonthContract(contractService.getContractCountByCreateTime(tokenInfo, lastMonthDate));

        dashboardVo.setTodayContract(contractService.getContractCountByCreateTime(tokenInfo, todayDate));
        dashboardVo.setYesterdayContract(contractService.getContractCountByCreateTime(tokenInfo, yesterdayDate));

        return ResultVo.success(dashboardVo);
    }
}
