package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.AttFile;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.mapper.AttFileMapper;
import com.jair.service.IAttFileService;
import com.jair.util.UploadUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.Date;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class AttFileServiceImpl extends ServiceImpl<AttFileMapper, AttFile> implements IAttFileService {

    @Autowired
    private UploadUtil uploadUtil;

    @Override
    public ResultVo upload(TokenInfo tokenInfo, MultipartFile multipartFile) throws Exception {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        String url = uploadUtil.upload(multipartFile);
        if (StringUtils.isEmpty(url)) {
            return ResultVo.businessError("上传文件失败");
        }

        AttFile attFile = new AttFile();
        attFile.setOriginName(multipartFile.getOriginalFilename());
        attFile.setFilePath(url);
        attFile.setFileType(0);
        attFile.setCreateTime(new Date());
        attFile.setCreateById(tokenInfo.getId());
        attFile.setCreateByName(tokenInfo.getRealName());
        attFile.setCompanyId(tokenInfo.getCompanyId());
        this.save(attFile);

        return ResultVo.success(attFile);
    }

    @Override
    public AttFile getAttFileById(Integer companyId, Integer fileId) {
        QueryWrapper<AttFile> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(AttFile::getId, fileId);
        queryWrapper.lambda().eq(AttFile::getCompanyId, companyId);
        return this.getOne(queryWrapper);
    }
}
