package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.jair.entity.bo.AttFile;
import com.jair.entity.bo.ContractSignAttachment;
import com.jair.mapper.ContractSignAttachmentMapper;
import com.jair.service.IAttFileService;
import com.jair.service.IContractSignAttachmentService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class ContractSignAttachmentServiceImpl extends ServiceImpl<ContractSignAttachmentMapper, ContractSignAttachment> implements IContractSignAttachmentService {

    @Autowired
    private IAttFileService fileService;

    @Override
    public List<AttFile> getFilesByContractId(Integer contractId) {
        QueryWrapper<ContractSignAttachment> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ContractSignAttachment::getContractId, contractId);
        List<ContractSignAttachment> list = list(queryWrapper);
        if (list == null || list.isEmpty()) {
            return null;
        }

        List<Integer> ids = list.stream().map(ContractSignAttachment::getFileId).collect(Collectors.toList());
        QueryWrapper<AttFile> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.lambda().in(AttFile::getId, ids);

        return fileService.list(queryWrapper1);
    }

    @Override
    public void deleteFilesByContractId(Integer contractId) {
        QueryWrapper<ContractSignAttachment> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ContractSignAttachment::getContractId, contractId);
        this.remove(queryWrapper);
    }

    @Override
    @Transactional
    public void updateContractFiles(List<Integer> files, Integer contractId) {
        this.deleteFilesByContractId(contractId);

        if (files == null || files.isEmpty()) {
            return;
        }
        for (Integer fileId : files) {
            ContractSignAttachment contractSignAttachment = new ContractSignAttachment();
            contractSignAttachment.setContractId(contractId);
            contractSignAttachment.setFileId(fileId);
            this.save(contractSignAttachment);
        }
    }
}
