package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.CfgData;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.DataTypeEnum;
import com.jair.entity.po.SearchDataPo;
import com.jair.entity.po.SysDataPo;
import com.jair.entity.vo.ModelVo;
import com.jair.mapper.CfgDataMapper;
import com.jair.service.IDataService;
import com.jair.util.BeanUtil;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class DataServiceImpl extends ServiceImpl<CfgDataMapper, CfgData> implements IDataService {

    @Override
    public ResultVo listEnvironmentalStandards(TokenInfo tokenInfo) {
        return ResultVo.success(listDatas(0, DataTypeEnum.ENVIRONMENTAL_STANDARDS.getKey(), null));
    }

    @Override
    public ResultVo listUseCharacters(TokenInfo tokenInfo) {
        return ResultVo.success(listDatas(0, DataTypeEnum.CHARACTER.getKey(), null));
    }

    @Override
    public ResultVo listColors(TokenInfo tokenInfo) {
        return ResultVo.success(listDatas(0, DataTypeEnum.COLOR.getKey(), null));
    }

    @Override
    public ResultVo listEngineTypes(TokenInfo tokenInfo) {
        return ResultVo.success(listDatas(0, DataTypeEnum.ENGINE_TYPE.getKey(), null));
    }

    @Override
    public ResultVo listVehicleTypes(TokenInfo tokenInfo) {
        return ResultVo.success(listDatas(0, DataTypeEnum.VEHICLE_TYPE.getKey(), null));
    }

    @Override
    public ResultVo listBrands(TokenInfo tokenInfo) {
        return ResultVo.success(listDatas(tokenInfo.getCompanyId(), DataTypeEnum.BRAND.getKey(), null));
    }

    public Integer getBrandIdByName(TokenInfo tokenInfo, String name) {
        QueryWrapper<CfgData> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CfgData::getCompanyId, tokenInfo.getCompanyId());
        queryWrapper.lambda().eq(CfgData::getType, DataTypeEnum.BRAND.getKey());
        queryWrapper.lambda().eq(CfgData::getName, name);
        if (this.getOne(queryWrapper) != null) {
            return this.getOne(queryWrapper).getId();
        }
        return 0;
    }

    @Override
    public ResultVo listModels(TokenInfo tokenInfo, SearchDataPo searchPo) {
        List<CfgData> datas = listDatas(tokenInfo.getCompanyId(), DataTypeEnum.MODEL.getKey(), searchPo.getParentId());
        if (datas == null || datas.isEmpty()) {
            return ResultVo.success();
        }

        List<ModelVo> models = new ArrayList<>();
        datas.forEach(data -> {
            CfgData brandData = getDataById(tokenInfo.getCompanyId(), data.getParentId());
            ModelVo modelVo = BeanUtil.transform(data, ModelVo.class);
            if (brandData != null) {
                modelVo.setBrandId(brandData.getId());
                modelVo.setBrandCode(brandData.getCode());
                modelVo.setBrandName(brandData.getName());
            }
            models.add(modelVo);
        });
        return ResultVo.success(models);
    }

    @Override
    public ResultVo createData(TokenInfo tokenInfo, SysDataPo dataPo) {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        if (isDataNameExists(tokenInfo.getCompanyId(), dataPo.getType(), dataPo.getParentId(), dataPo.getName())) {
            return ResultVo.businessError("该数据已存在");
        }
        CfgData data = BeanUtil.transform(dataPo, CfgData.class);
        data.setCompanyId(tokenInfo.getCompanyId());
        this.save(data);

        return ResultVo.success();
    }

    @Override
    public List<CfgData> listDatas(Integer companyId, Integer type, Integer parentId) {
        QueryWrapper<CfgData> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CfgData::getCompanyId, companyId);
        queryWrapper.lambda().eq(CfgData::getType, type);
        if (parentId != null) {
            queryWrapper.lambda().eq(CfgData::getParentId, parentId);
        }
        return this.list(queryWrapper);
    }

    @Override
    public CfgData getDataById(Integer companyId, Integer dataId) {
        QueryWrapper<CfgData> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CfgData::getCompanyId, companyId);
        queryWrapper.lambda().eq(CfgData::getId, dataId);
        return this.getOne(queryWrapper);
    }

    @Override
    public Boolean isDataNameExists(Integer companyId, Integer type, Integer parentId, String name) {
        QueryWrapper<CfgData> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CfgData::getCompanyId, companyId);
        queryWrapper.lambda().eq(CfgData::getName, name);
        if (parentId != null)
            queryWrapper.lambda().eq(CfgData::getParentId, parentId);
        if (type != null)
            queryWrapper.lambda().eq(CfgData::getType, type);
        return this.getOne(queryWrapper) != null;
    }
}
