package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.AttFile;
import com.jair.entity.bo.Driver;
import com.jair.entity.bo.DrivingLicense;
import com.jair.entity.bo.IdentityCard;
import com.jair.entity.common.PageInfo;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.AccountTypeEnum;
import com.jair.entity.po.DriverPo;
import com.jair.entity.po.SearchDriverPo;
import com.jair.entity.po.ValidateResult;
import com.jair.entity.vo.ContractVo;
import com.jair.entity.vo.DriverVo;
import com.jair.entity.vo.VehicleVo;
import com.jair.mapper.DriverMapper;
import com.jair.service.IAttFileService;
import com.jair.service.IDriverService;
import com.jair.service.IDrivingLicenseService;
import com.jair.service.IIdentityCardService;
import com.jair.util.BeanUtil;
import com.jair.util.DateUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class DriverServiceImpl extends ServiceImpl<DriverMapper, Driver> implements IDriverService {
    @Autowired
    private IIdentityCardService identityCardService;
    @Autowired
    private IDrivingLicenseService licenseService;
    @Autowired
    private IAttFileService fileService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo createDriver(TokenInfo tokenInfo, DriverPo driverPo) throws Exception {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        ValidateResult validate = driverPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }
        if (isDriverExists(tokenInfo.getCompanyId(), null, driverPo.getIdentityNumber())) {
            return ResultVo.businessError("身份证号码已经存在");
        }
        Date createTime = new Date();

//        String identityNumber = driverPo.getIdentityNumber();
//        String brithday = null;
//        if (identityNumber.trim().length() == 15) {
//            brithday = identityNumber.substring(6, 12);
//            Integer yy = Integer.parseInt(brithday.substring(0, 2));
//            brithday = yy < 40 ? "20" : "19" + brithday;
//        } else {
//            brithday = identityNumber.substring(6, 14);
//        }

        IdentityCard identity = new IdentityCard();
        identity.setBirthday(driverPo.getBirthday());
        identity.setNation(driverPo.getNation());
        identity.setOriginAddress(driverPo.getOriginAddress());
        identity.setIssueOrganization(driverPo.getIssueOrganization());
        identity.setValidPeriodStart(driverPo.getIdentityValidPeriodStart());
        identity.setValidPeriodEnd(driverPo.getIdentityValidPeriodEnd());
        identity.setFrontFileId(driverPo.getFrontFileId());
        identity.setBackFileId(driverPo.getBackFileId());
        identity.setCreateTime(createTime);
        identity.setCreateById(tokenInfo.getId());
        identity.setCreateByName(tokenInfo.getRealName());
        identity.setCompanyId(tokenInfo.getCompanyId());
        identityCardService.save(identity);
        if (identity.getId() == null || identity.getId() == 0) {
            throw new Exception("身份证信息保存失败");
        }

        DrivingLicense license = new DrivingLicense();
        license.setCredentialNo(driverPo.getCredentialNo());
        license.setFileNo(driverPo.getFileNo());
        license.setFirstIssue(driverPo.getFirstIssue());
        license.setDrivingClass(driverPo.getDrivingClass());
        license.setValidPeriodStart(driverPo.getLicenseValidPeriodStart());
        license.setValidPeriodEnd(driverPo.getLicenseValidPeriodEnd());
        license.setMainFileId(driverPo.getMainFileId());
        license.setSecondFileId(driverPo.getSecondFileId());
        license.setCreateTime(createTime);
        license.setCreateById(tokenInfo.getId());
        license.setCreateByName(tokenInfo.getRealName());
        license.setCompanyId(tokenInfo.getCompanyId());
        licenseService.save(license);
        if (license.getId() == null || license.getId() == 0) {
            throw new Exception("驾驶证信息保存失败");
        }

        Driver driver = BeanUtil.transform(driverPo, Driver.class);
        driver.setIdentityId(identity.getId());
        driver.setLicenseId(license.getId());
        driver.setCreateTime(createTime);
        driver.setCreateById(tokenInfo.getId());
        driver.setCreateByName(tokenInfo.getRealName());
        driver.setCompanyId(tokenInfo.getCompanyId());
        this.save(driver);
        if (driver.getId() == null || driver.getId() == 0) {
            throw new Exception("司机信息保存失败");
        }

        return ResultVo.success(driver);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo updateDriver(TokenInfo tokenInfo, DriverPo driverPo) throws Exception {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        ValidateResult validate = driverPo.validate();
        if (!validate.isSucceed()) {
            return ResultVo.businessError(validate.getData());
        }

        Driver driver = getDriverById(tokenInfo.getCompanyId(), driverPo.getId());
        if (driver == null) {
            return ResultVo.businessError("未找到该司机");
        }

//        String identityNumber = driverPo.getIdentityNumber();
//        String brithday = null;
//        if (identityNumber.trim().length() == 15) {
//            brithday = identityNumber.substring(6, 12);
//            Integer yy = Integer.parseInt(brithday.substring(0, 2));
//            brithday = yy < 40 ? "20" : "19" + brithday;
//        } else {
//            brithday = identityNumber.substring(6, 14);
//        }

        IdentityCard identity = identityCardService.getById(driver.getIdentityId());
        identity.setBirthday(driverPo.getBirthday());
        identity.setNation(driverPo.getNation());
        identity.setOriginAddress(driverPo.getOriginAddress());
        identity.setIssueOrganization(driverPo.getIssueOrganization());
        identity.setValidPeriodStart(driverPo.getIdentityValidPeriodStart());
        identity.setValidPeriodEnd(driverPo.getIdentityValidPeriodEnd());
        identity.setFrontFileId(driverPo.getFrontFileId());
        identity.setBackFileId(driverPo.getBackFileId());
        identityCardService.updateById(identity);

        DrivingLicense license = licenseService.getById(driver.getLicenseId());
        license.setCredentialNo(driverPo.getCredentialNo());
        license.setFileNo(driverPo.getFileNo());
        license.setFirstIssue(driverPo.getFirstIssue());
        license.setDrivingClass(driverPo.getDrivingClass());
        license.setValidPeriodStart(driverPo.getLicenseValidPeriodStart());
        license.setValidPeriodEnd(driverPo.getLicenseValidPeriodEnd());
        license.setMainFileId(driverPo.getMainFileId());
        license.setSecondFileId(driverPo.getSecondFileId());
        licenseService.updateById(license);

        BeanUtil.copyProperties(driverPo, driver);
        this.updateById(driver);

        return ResultVo.success();
    }

    @Override
    public ResultVo listDrivers(TokenInfo tokenInfo, PageInfo<SearchDriverPo> pageInfo) {
        pageInfo = PageInfo.checkParam(pageInfo);
        SearchDriverPo searchPo = pageInfo.getRecords();

        QueryWrapper<Driver> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Driver::getCompanyId, tokenInfo.getCompanyId());
        }
        if (!StringUtils.isEmpty(searchPo.getIdentityNumber())) {
            queryWrapper.lambda().like(Driver::getIdentityNumber, searchPo.getIdentityNumber());
        }
        if (!StringUtils.isEmpty(searchPo.getName())) {
            queryWrapper.lambda().like(Driver::getName, searchPo.getName());
        }
        if (!StringUtils.isEmpty(searchPo.getPhone())) {
            queryWrapper.lambda().like(Driver::getPhone, searchPo.getPhone());
        }
        if (!StringUtils.isEmpty(searchPo.getAddress())) {
            queryWrapper.lambda().like(Driver::getAddress, searchPo.getAddress());
        }
        if (!StringUtils.isEmpty(searchPo.getContactName())) {
            queryWrapper.lambda().like(Driver::getContactName, searchPo.getContactName());
        }
        if (!StringUtils.isEmpty(searchPo.getContactPhone())) {
            queryWrapper.lambda().like(Driver::getContactPhone, searchPo.getContactPhone());
        }

        List<DriverVo> driverVos = new ArrayList<>();

        Page<Driver> page = new Page<>(pageInfo.getCurrent(), pageInfo.getSize());
        IPage<Driver> selectPage = this.page(page, queryWrapper);
        if (selectPage.getRecords() != null) {
            selectPage.getRecords().forEach(driver -> {
                // TODO 获取 合同状态
                DriverVo driverVo = BeanUtil.transform(driver, DriverVo.class);

                driverVo = this.fillIdentityCardInfo(tokenInfo, driverVo, driver.getIdentityId());
                driverVo = this.fillDrivingLicenseInfo(tokenInfo, driverVo, driver.getLicenseId());

                driverVos.add(driverVo);
            });
        }

        PageInfo<List<DriverVo>> resultPageInfo = new PageInfo<>();
        resultPageInfo.setCurrent(pageInfo.getCurrent());
        resultPageInfo.setSize(pageInfo.getSize());
        resultPageInfo.setTotal(selectPage.getTotal());
        resultPageInfo.setRecords(driverVos);

        return ResultVo.success(resultPageInfo);
    }

    @Override
    public ResultVo listAllDrivers(TokenInfo tokenInfo) {
        QueryWrapper<Driver> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().select(Driver::getId, Driver::getName, Driver::getPhone);
        queryWrapper.lambda().eq(Driver::getCompanyId, tokenInfo.getCompanyId());

        List<Driver> drivers = this.list(queryWrapper);
        if (drivers == null || drivers.isEmpty()) {
            return ResultVo.success();
        }
        List<DriverVo> driverVos = new ArrayList<>();
        drivers.forEach(driver -> {
            DriverVo driverVo = BeanUtil.transform(driver, DriverVo.class);
            driverVos.add(driverVo);
        });
        return ResultVo.success(driverVos);
    }

    @Override
    public boolean isDriverExists(Integer companyId, Integer driverId, String identityNumber) {
        QueryWrapper<Driver> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Driver::getCompanyId, companyId);
        queryWrapper.lambda().eq(Driver::getIdentityNumber, identityNumber);
        if (driverId != null) {
            queryWrapper.lambda().ne(Driver::getId, driverId);
        }
        if (this.getOne(queryWrapper) == null) {
            return false;
        }
        return true;
    }

    @Override
    public ResultVo getDriverById(TokenInfo tokenInfo, Integer driverId) {
        Driver driver = this.getDriverById(tokenInfo.getCompanyId(), driverId);
        DriverVo driverVo = this.translateDriverToVo(tokenInfo, driver);
        return ResultVo.success(driverVo);
    }

    @Override
    public Driver getDriverById(Integer companyId, Integer driverId) {
        QueryWrapper<Driver> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Driver::getCompanyId, companyId);
        queryWrapper.lambda().eq(Driver::getId, driverId);
        return this.getOne(queryWrapper);
    }

    @Override
    public DriverVo fillIdentityCardInfo(TokenInfo tokenInfo, DriverVo driverVo, Integer identityId) {
        IdentityCard identityCard = identityCardService.getById(identityId);
        if (identityCard != null) {
            driverVo.setNation(identityCard.getNation());
            driverVo.setBirthday(identityCard.getBirthday());
            driverVo.setOriginAddress(identityCard.getOriginAddress());
            driverVo.setIssueOrganization(identityCard.getIssueOrganization());
            driverVo.setIdentityValidPeriodStart(identityCard.getValidPeriodStart());
            driverVo.setIdentityValidPeriodEnd(identityCard.getValidPeriodEnd());
            if (identityCard.getFrontFileId() != null) {
                driverVo.setFrontFile(fileService.getAttFileById(tokenInfo.getCompanyId(), identityCard.getFrontFileId()));
            }
            if (identityCard.getBackFileId() != null) {
                driverVo.setBackFile(fileService.getAttFileById(tokenInfo.getCompanyId(), identityCard.getBackFileId()));
            }
        }
        return driverVo;
    }

    @Override
    public DriverVo fillDrivingLicenseInfo(TokenInfo tokenInfo, DriverVo driverVo, Integer licenseId) {
        DrivingLicense license = licenseService.getById(licenseId);
        if (license != null) {
            driverVo.setCredentialNo(license.getCredentialNo());
            driverVo.setFileNo(license.getFileNo());
            driverVo.setFirstIssue(license.getFirstIssue());
            driverVo.setDrivingClass(license.getDrivingClass());
            driverVo.setLicenseValidPeriodStart(license.getValidPeriodStart());
            driverVo.setLicenseValidPeriodEnd(license.getValidPeriodEnd());
            if (license.getMainFileId() != null) {
                driverVo.setMainFile(fileService.getAttFileById(tokenInfo.getCompanyId(), license.getMainFileId()));
            }
            if (license.getSecondFileId() != null) {
                driverVo.setSecondFile(fileService.getAttFileById(tokenInfo.getCompanyId(), license.getSecondFileId()));
            }
        }
        return driverVo;
    }

    @Override
    public DriverVo translateDriverToVo(TokenInfo tokenInfo, Driver driver) {
        DriverVo driverVo = BeanUtil.transform(driver, DriverVo.class);
        driverVo = this.fillIdentityCardInfo(tokenInfo, driverVo, driver.getIdentityId());
        driverVo = this.fillDrivingLicenseInfo(tokenInfo, driverVo, driver.getLicenseId());
        return driverVo;
    }

    @Override
    public Integer getAllDriverCount(TokenInfo tokenInfo) {
        QueryWrapper<Driver> queryWrapper = new QueryWrapper<>();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            queryWrapper.lambda().eq(Driver::getCompanyId, tokenInfo.getCompanyId());
        }
        return this.count(queryWrapper);
    }

}
