package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.CfgData;
import com.jair.entity.bo.CfgFee;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.mapper.CfgDataMapper;
import com.jair.mapper.CfgFeeMapper;
import com.jair.service.IFeeService;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class FeeServiceImpl extends ServiceImpl<CfgFeeMapper, CfgFee> implements IFeeService {

    @Override
    public ResultVo list(TokenInfo tokenInfo, Integer status) {
        QueryWrapper<CfgFee> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CfgFee::getCompanyId, tokenInfo.getCompanyId());
        if (status == 0 || status == 1) {
            queryWrapper.lambda().eq(CfgFee::getStatus, status);
        }
        return ResultVo.success(this.list(queryWrapper));
    }

    @Override
    public ResultVo create(TokenInfo tokenInfo, CfgFee fee) {
        if (fee == null) {
            return ResultVo.businessError("数据异常");
        }
        if (StringUtils.isEmpty(fee.getName())) {
            return ResultVo.businessError("费用名称不能为空");
        }
        if (fee.getAmount() <= 0) {
            return ResultVo.businessError("费用金额不能为0");
        }
        if (fee.getTiming() > 5 || fee.getTiming() < 1) {
            return ResultVo.businessError("费用时机不正确");
        }
        fee.setCompanyId(tokenInfo.getCompanyId());
        fee.setStatus(1);
        this.save(fee);
        return ResultVo.success();
    }

    @Override
    public ResultVo disable(TokenInfo tokenInfo, Integer feeId) {
        CfgFee fee = this.getFeeById(tokenInfo.getCompanyId(), feeId);
        fee.setStatus(0);
        this.updateById(fee);
        return ResultVo.success();
    }

    @Override
    public ResultVo enable(TokenInfo tokenInfo, Integer feeId) {
        CfgFee fee = this.getFeeById(tokenInfo.getCompanyId(), feeId);
        fee.setStatus(1);
        this.updateById(fee);
        return ResultVo.success();
    }

    @Override
    public CfgFee getFeeById(Integer companyId, Integer feeId) {
        QueryWrapper<CfgFee> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CfgFee::getCompanyId, companyId);
        queryWrapper.lambda().eq(CfgFee::getId, feeId);

        return this.getOne(queryWrapper);
    }


}
