package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.*;
import com.jair.entity.common.PageInfo;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.constant.*;
import com.jair.entity.po.*;
import com.jair.entity.vo.FeePlanningVo;
import com.jair.entity.vo.PlanningOperationLogVo;
import com.jair.mapper.ContractMapper;
import com.jair.mapper.FeePlanningMapper;
import com.jair.mapper.PlanningOperationLogMapper;
import com.jair.service.IContractService;
import com.jair.service.IPlanningService;
import com.jair.util.DateUtil;
import com.jair.util.StringAppend;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class PlanningServiceImpl extends ServiceImpl<FeePlanningMapper, FeePlanning> implements IPlanningService {
    @Autowired
    private FeePlanningMapper planningMapper;
    @Autowired
    private PlanningOperationLogMapper operationLogMapper;
    @Autowired
    @Lazy
    private IContractService contractService;

    @Override
    public ResultVo listPlanning(TokenInfo tokenInfo, PageInfo<SearchPlanningPo> pageInfo) {
        pageInfo = PageInfo.checkParam(pageInfo);
        SearchPlanningPo searchPo = pageInfo.getRecords();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            searchPo.setCompanyId(tokenInfo.getCompanyId());
        }

        Page<FeePlanningVo> page = new Page<>(pageInfo.getCurrent(), pageInfo.getSize());
        IPage<FeePlanningVo> selectPage = this.planningMapper.getDriverPlannings(page, searchPo);

        PageInfo<List<FeePlanningVo>> resultPageInfo = new PageInfo<>();
        resultPageInfo.setCurrent(pageInfo.getCurrent());
        resultPageInfo.setSize(pageInfo.getSize());
        resultPageInfo.setTotal(selectPage.getTotal());
        resultPageInfo.setRecords(selectPage.getRecords());

        return ResultVo.success(resultPageInfo);
    }

    @Override
    public ResultVo listPlanningByContractId(TokenInfo tokenInfo, Integer contractId, Integer type) {
        QueryWrapper<FeePlanning> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(FeePlanning::getCompanyId, tokenInfo.getCompanyId());
        queryWrapper.lambda().eq(FeePlanning::getContractId, contractId);
        if (type == 1) {
            queryWrapper.lambda().eq(FeePlanning::getType, type);
        } else {
            queryWrapper.lambda().in(FeePlanning::getType, 2, 3);
        }
        queryWrapper.lambda().orderByAsc(FeePlanning::getName, FeePlanning::getPlanDate, FeePlanning::getId);
        return ResultVo.success(this.list(queryWrapper));
    }


    @Override
    public void saveFeePlanning(TokenInfo tokenInfo, Contract contract, List<CfgFeePo> fees) throws Exception {
        if (fees == null || fees.isEmpty()) {
            return;
        }

        for (CfgFeePo fee : fees) {
            Date planningDate = contract.getStartDate();
            Integer paidAmount = fee.getFirstPaidAmount();
            if (paidAmount == null) {
                paidAmount = 0;
            }
            while (true) {
                FeePlanning planning = new FeePlanning();
                planning.setType(2); // 费用项
                planning.setName(fee.getName());
                planning.setContractId(contract.getId());
                planning.setContractType(contract.getType());
                planning.setPlanDate(planningDate);
                planning.setPayableAmount(fee.getAmount());
                planning.setPaidAmount(0);
                planning.setPaymentStatus(0); // 未付

                planning.setCreateTime(new Date());
                planning.setCreateById(tokenInfo.getId());
                planning.setCreateByName(tokenInfo.getRealName());
                planning.setCompanyId(tokenInfo.getCompanyId());
                planning.setStatus(1);

                this.save(planning);

                // 生成付款日志
                if (contract.getType() == ContractTypeEnum.DRIVER_CONTRACT.getKey() && paidAmount > 0) {
                    PayingPo payingPo = new PayingPo();
                    payingPo.setPlanningId(planning.getId());
                    payingPo.setAmount(paidAmount);
                    payingPo.setRemark("创建付款");
                    this.paying(tokenInfo, payingPo);
                }

                paidAmount = 0; // 仅首月
                if (fee.getTiming() == FeeTimingEnum.ONCE.getKey()) { // 一次性费用
                    break;
                } else if (fee.getTiming() == FeeTimingEnum.WEEK.getKey()) { // 周度费用

                    planningDate = DateUtil.getAddTime(planningDate, Calendar.DATE, 7);
                } else if (fee.getTiming() == FeeTimingEnum.MONTH.getKey()) { // 月度费用

                    if (contract.getLeaseType() == LeaseTypeEnum.MONTH.getKey()) {
                        planningDate = DateUtil.getAddTime(planningDate, Calendar.MONTH, 1);
                    } else {
                        planningDate = DateUtil.getAddTime(planningDate, Calendar.DATE, 30);
                    }
                } else if (fee.getTiming() == FeeTimingEnum.QUARTER.getKey()) { // 季度费用

                    if (contract.getLeaseType() == LeaseTypeEnum.MONTH.getKey()) {
                        planningDate = DateUtil.getAddTime(planningDate, Calendar.MONTH, 3);
                    } else {
                        planningDate = DateUtil.getAddTime(planningDate, Calendar.DATE, 30 * 3);
                    }
                } else if (fee.getTiming() == FeeTimingEnum.YEAR.getKey()) { // 年度费用

                    if (contract.getLeaseType() == LeaseTypeEnum.MONTH.getKey()) {
                        planningDate = DateUtil.getAddTime(planningDate, Calendar.YEAR, 1);
                    } else {
                        planningDate = DateUtil.getAddTime(planningDate, Calendar.DATE, 30 * 12);
                    }
                }

                if (planningDate.getTime() >= contract.getEndDate().getTime()) {
                    break;
                }

            }
        }
    }

    @Override
    public void saveRentPlanning(TokenInfo tokenInfo, Contract contract, Integer firstPaidAmount) throws Exception {
        Date planningDate = contract.getStartDate();
        Integer paidAmount = firstPaidAmount;
        while (true) {
            FeePlanning planning = new FeePlanning();
            planning.setType(1); // 租金
            planning.setName("租金"); // 租金
            planning.setContractId(contract.getId());
            planning.setContractType(contract.getType());
            planning.setPlanDate(planningDate);
            planning.setPayableAmount(contract.getRentAmount());
            planning.setPaidAmount(0);
            planning.setPaymentStatus(0); // 未付

            planning.setCreateTime(new Date());
            planning.setCreateById(tokenInfo.getId());
            planning.setCreateByName(tokenInfo.getRealName());
            planning.setCompanyId(tokenInfo.getCompanyId());
            planning.setStatus(1);

            this.save(planning);

            // 生成付款日志
            if (contract.getType() == ContractTypeEnum.DRIVER_CONTRACT.getKey() && paidAmount > 0) {
                PayingPo payingPo = new PayingPo();
                payingPo.setPlanningId(planning.getId());
                payingPo.setAmount(paidAmount);
                payingPo.setRemark("创建付款");
                this.paying(tokenInfo, payingPo);
            }

//            System.out.println("plan:" + planning.toString());
            paidAmount = 0; // 仅首月
            if (contract.getLeaseType() == LeaseTypeEnum.DAY_OF_MONTH.getKey()) {
                planningDate = DateUtil.getAddTime(planningDate, Calendar.DATE, 30);
            } else if (contract.getLeaseType() == LeaseTypeEnum.MONTH.getKey()) {
                planningDate = DateUtil.getAddTime(planningDate, Calendar.MONTH, 1);
            } else if (contract.getLeaseType() == LeaseTypeEnum.WEEK.getKey()) {
                planningDate = DateUtil.getAddTime(planningDate, Calendar.DATE, 7);
            } else {
                break;
            }
            if (planningDate.getTime() >= contract.getEndDate().getTime()) {
                break;
            }
        }
    }

    @Override
    public void saveDepositPlanning(TokenInfo tokenInfo, Contract contract) throws Exception {
        Integer amount = contract.getPayableDepositAmount() - contract.getPaidDepositAmount();
        if (amount <= 0) {
            return;
        }
        Date planningDate = contract.getStartDate();

        FeePlanning planning = new FeePlanning();
        planning.setType(3); // 押金
        planning.setName("押金"); // 押金
        planning.setContractId(contract.getId());
        planning.setContractType(contract.getType());
        planning.setPlanDate(planningDate);
        planning.setPayableAmount(amount);
        planning.setPaidAmount(0);
        planning.setPaymentStatus(0); // 未付

        planning.setCreateTime(new Date());
        planning.setCreateById(tokenInfo.getId());
        planning.setCreateByName(tokenInfo.getRealName());
        planning.setCompanyId(tokenInfo.getCompanyId());
        planning.setStatus(1);

        this.save(planning);
    }

    @Override
    public void disablePlanningByContractId(Integer contractId) {
        planningMapper.disablePlanningByContractId(contractId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo paying(TokenInfo tokenInfo, PayingPo payingPo) throws Exception {
        if (payingPo.getAmount() <= 0) {
//            return ResultVo.businessError("付款金额不正确");
            throw new Exception("付款金额不正确");
        }
        FeePlanning planning = this.getById(payingPo.getPlanningId());
        if (planning == null) {
//            return ResultVo.businessError("该计划不存在");
            throw new Exception("该计划不存在");
        }
        if (planning.getCompanyId() != tokenInfo.getCompanyId()) {
//            return ResultVo.businessError("权限不足");
            throw new Exception("权限不足");
        }

        planning.setPaidAmount(planning.getPaidAmount() + payingPo.getAmount());
        if (planning.getPaidAmount() < planning.getPayableAmount()) {
            planning.setPaymentStatus(1); // 已付部分
        } else {
            planning.setPaymentStatus(2); // 已付全部
        }
        this.updateById(planning);

        if (planning.getType() == 3) {//押金 更新合同押金状态
            Contract contract = this.contractService.getContractById(tokenInfo.getCompanyId(), planning.getContractId());
            if (contract != null) {
                contract.setPaidDepositAmount(contract.getPaidDepositAmount() + payingPo.getAmount());
                if (contract.getPaidDepositAmount() >= contract.getPayableDepositAmount()) {
                    contract.setDepositStatus(DepositStatusEnum.NORMAL.getKey());
                }
                this.contractService.updateById(contract);
            }
        }

        this.makePayingLog(tokenInfo, planning, payingPo);

        return ResultVo.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultVo delay(TokenInfo tokenInfo, DelayPo delayPo) {
        FeePlanning planning = this.getById(delayPo.getPlanningId());
        if (planning == null) {
            return ResultVo.businessError("该计划不存在");
        }
        if (planning.getCompanyId() != tokenInfo.getCompanyId()) {
            return ResultVo.businessError("权限不足");
        }
        planning.setDelayDate(delayPo.getDelayDate());
        this.updateById(planning);

        this.makeDelayLog(tokenInfo, planning, delayPo);

        return ResultVo.success();
    }

    @Override
    public ResultVo listLogs(TokenInfo tokenInfo, PageInfo<SearchPlanningLogPo> pageInfo) {
        pageInfo = PageInfo.checkParam(pageInfo);
        SearchPlanningLogPo searchPo = pageInfo.getRecords();
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            searchPo.setCompanyId(tokenInfo.getCompanyId());
        }

        Page<PlanningOperationLogVo> page = new Page<>(pageInfo.getCurrent(), pageInfo.getSize());
        IPage<PlanningOperationLogVo> selectPage = this.operationLogMapper.queryLogs(page, searchPo);

        PageInfo<List<PlanningOperationLogVo>> resultPageInfo = new PageInfo<>();
        resultPageInfo.setCurrent(pageInfo.getCurrent());
        resultPageInfo.setSize(pageInfo.getSize());
        resultPageInfo.setTotal(selectPage.getTotal());
        resultPageInfo.setRecords(selectPage.getRecords());

        return ResultVo.success(resultPageInfo);
    }

    private void makePayingLog(TokenInfo tokenInfo, FeePlanning planning, PayingPo payingPo) {
        PlanningOperationLog operationLog = this.makeLog(tokenInfo, planning);
        operationLog.setType(PlanningLogActionEnum.PAYING.getType());
        operationLog.setAction(PlanningLogActionEnum.PAYING.getName());
        operationLog.setAmount(payingPo.getAmount());
        operationLog.setRemark(payingPo.getRemark());
        operationLogMapper.insert(operationLog);
    }

    private void makeDelayLog(TokenInfo tokenInfo, FeePlanning planning, DelayPo delayPo) {
        PlanningOperationLog operationLog = this.makeLog(tokenInfo, planning);
        operationLog.setType(PlanningLogActionEnum.DELAY.getType());
        operationLog.setAction(PlanningLogActionEnum.DELAY.getName());
        StringAppend stringAppend = new StringAppend("，");
        stringAppend.append(String.format("将延期日期修改为[%s]", DateUtil.format2Date(delayPo.getDelayDate())));
        stringAppend.append(delayPo.getRemark());
        operationLog.setRemark(stringAppend.toString());
        operationLogMapper.insert(operationLog);
    }

    private PlanningOperationLog makeLog(TokenInfo tokenInfo, FeePlanning planning) {
        PlanningOperationLog operationLog = new PlanningOperationLog();
        operationLog.setContractId(planning.getContractId());
        operationLog.setPlanningId(planning.getId());


        operationLog.setCreateById(tokenInfo.getId());
        operationLog.setCreateByName(tokenInfo.getRealName());
        operationLog.setCreateTime(new Date());
        operationLog.setCompanyId(tokenInfo.getCompanyId());

        return operationLog;
    }

    @Override
    public Integer getPlanAmountByPlanDate(TokenInfo tokenInfo, String date) {
        Integer companyId = null;
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            companyId = tokenInfo.getCompanyId();
        }
        return planningMapper.getPlanAmountByPlanDate(companyId, date);
    }

    @Override
    public Integer getActualAmountByCreateTime(TokenInfo tokenInfo, String date) {
        Integer companyId = null;
        if (tokenInfo.getType() != AccountTypeEnum.ADMIN.getKey()) {
            companyId = tokenInfo.getCompanyId();
        }
        return planningMapper.getActualAmountByCreateTime(companyId, date);
    }
}
