package com.jair.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jair.entity.bo.Source;
import com.jair.entity.common.ResultVo;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.po.SourcePo;
import com.jair.mapper.SourceMapper;
import com.jair.service.ISourceService;
import com.jair.util.BeanUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Date;

/**
 * <p>
 * 车源信息 服务实现类
 * </p>
 *
 * @author Jair
 * @since 2020-08-25
 */
@Service
public class SourceServiceImpl extends ServiceImpl<SourceMapper, Source> implements ISourceService {

    @Override
    public ResultVo createSource(TokenInfo tokenInfo, SourcePo sourcePo) {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        if (StringUtils.isEmpty(sourcePo.getName())) {
            return ResultVo.businessError("车源名称不能为空");
        }
        if (isSourceNameExists(tokenInfo.getCompanyId(), sourcePo.getId(), sourcePo.getName())) {
            return ResultVo.businessError("车源名称已经存在");
        }

        Source source = BeanUtil.transform(sourcePo, Source.class);
        source.setCreateTime(new Date());
        source.setCreateById(tokenInfo.getId());
        source.setCreateByName(tokenInfo.getRealName());
        source.setCompanyId(tokenInfo.getCompanyId());
        source.setStatus(1);

        this.save(source);
        return ResultVo.success();
    }

    @Override
    public ResultVo updateSource(TokenInfo tokenInfo, SourcePo sourcePo) {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        if (StringUtils.isEmpty(sourcePo.getName())) {
            return ResultVo.businessError("车源名称不能为空");
        }
        if (isSourceNameExists(tokenInfo.getCompanyId(), sourcePo.getId(), sourcePo.getName())) {
            return ResultVo.businessError("车源名称已经存在");
        }

        Source source = this.getSourceById(tokenInfo.getCompanyId(), sourcePo.getId());
        BeanUtil.copyProperties(sourcePo, source);

        this.updateById(source);
        return ResultVo.success();
    }

    @Override
    public ResultVo updateSourceStatus(TokenInfo tokenInfo, SourcePo sourcePo) {
        if (tokenInfo.getCompanyId() == null || tokenInfo.getCompanyId() <= 0) {
            return ResultVo.businessError("未知的企业帐号");
        }
        Source source = this.getSourceById(tokenInfo.getCompanyId(), sourcePo.getId());
        source.setStatus(sourcePo.getStatus());

        this.updateById(source);
        return ResultVo.success();
    }

    @Override
    public ResultVo listSources(TokenInfo tokenInfo, SourcePo sourcePo) {
        QueryWrapper<Source> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Source::getCompanyId, tokenInfo.getCompanyId());
        if (sourcePo != null) {
            if (!StringUtils.isEmpty(sourcePo.getName()))
                queryWrapper.lambda().like(Source::getName, sourcePo.getName());
        }
        return ResultVo.success(this.list(queryWrapper));
    }

    @Override
    public boolean isSourceNameExists(Integer companyId, Integer id, String sourceName) {
        QueryWrapper<Source> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Source::getCompanyId, companyId);
        queryWrapper.lambda().eq(Source::getName, sourceName);
        if (id != null && id != 0) {
            queryWrapper.lambda().ne(Source::getId, id);
        }
        Source source = this.getOne(queryWrapper);
        if (source == null) {
            return false;
        }
        return true;
    }

    @Override
    public Source getSourceById(Integer companyId, Integer sourceId) {
        QueryWrapper<Source> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(Source::getId, sourceId);
        queryWrapper.lambda().eq(Source::getCompanyId, companyId);
        return this.getOne(queryWrapper);
    }
}
