package com.jair.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.jair.entity.bo.VehicleLicense;
import com.jair.entity.vo.DrivingLicenseVo;
import com.jair.entity.vo.IdentityCardVo;
import com.jair.entity.vo.VehicleLicenseDetectVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Base64;
import java.util.List;

@Service
@Slf4j
public class BaiduAI {
    private static final String BASE_URL = "https://aip.baidubce.com";
    private static final String URL_BAIDU_TOKEN = BASE_URL + "/oauth/2.0/token?grant_type=client_credentials&client_id=%s&client_secret=%s";
    private static final String URL_BAIDU_TEXT_DETECT = BASE_URL + "/rest/2.0/ocr/v1/general_basic?charset=UTF-8&access_token=%s";
    private static final String URL_BAIDU_VEHICLE_LICENSE = BASE_URL + "/rest/2.0/ocr/v1/vehicle_license?access_token=%s";
    private static final String URL_BAIDU_IDCARD = BASE_URL + "/rest/2.0/ocr/v1/idcard?access_token=%s";
    private static final String URL_BAIDU_DRIVER_LICENSE = BASE_URL + "/rest/2.0/ocr/v1/driving_license?access_token=%s";

    private static final String APP_ID = "18029293";
    private static final String APP_KEY = "kibweDD5PsLfu9exll0pMVet";
    private static final String SECRET_KEY = "RwBcY9mbPLLhLqWMwjA7LMYEfnGSVsLS";


    public VehicleLicenseDetectVo vehicleLicenseDetect(String imageUrl, boolean isMain) throws Exception {
        return vehicleLicenseDetect(getBytesByUrl(imageUrl), isMain);
    }

    public VehicleLicenseDetectVo vehicleLicenseDetect(File image, boolean isMain) throws Exception {
        FileInputStream fis = new FileInputStream(image);
        byte[] bytes = readNBytes(fis);
        return vehicleLicenseDetect(bytes, isMain);
    }

    public IdentityCardVo identityCardDetect(String imageUrl, boolean isMain) throws Exception {
        return identityCardDetect(getBytesByUrl(imageUrl), isMain);
    }

    public IdentityCardVo identityCardDetect(File image, boolean isMain) throws Exception {
        FileInputStream fis = new FileInputStream(image);
        byte[] bytes = readNBytes(fis);
        return identityCardDetect(bytes, isMain);
    }

    public DrivingLicenseVo drivingLicenseDetect(String imageUrl) throws Exception {
        return drivingLicenseDetect(getBytesByUrl(imageUrl));
    }

    public DrivingLicenseVo drivingLicenseDetect(File image) throws Exception {
        FileInputStream fis = new FileInputStream(image);
        byte[] bytes = readNBytes(fis);
        return drivingLicenseDetect(bytes);
    }


    public VehicleLicenseDetectVo vehicleLicenseDetect(byte[] imageBytes, boolean isMain) throws Exception {
        String accessToken = getAccessToken();

        String base64 = Base64.getEncoder().encodeToString(imageBytes);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> parameters = new LinkedMultiValueMap<>();
        parameters.add("image", base64);
        parameters.add("detect_direction", "true");
        parameters.add("vehicle_license_side", isMain ? "front" : "back");
        parameters.add("unified", "true");

//        System.out.println(base64);
        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<MultiValueMap> requestEntity = new HttpEntity<>(parameters, headers);
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(String.format(URL_BAIDU_VEHICLE_LICENSE, accessToken), requestEntity, String.class);

//        {"log_id": 3374241317740710780, "words_result_num": 1, "words_result": [{"words": "m AL"}], "language": 0}
        String body = responseEntity.getBody();
        System.out.println(body);
        JSONObject wordsResult = JSON.parseObject(body).getJSONObject("words_result");
        VehicleLicenseDetectVo license = new VehicleLicenseDetectVo();

        license.setPlateNo(wordsResult.getJSONObject("号牌号码") != null ? wordsResult.getJSONObject("号牌号码").getString("words") : null);
        license.setVin(wordsResult.getJSONObject("车辆识别代号") != null ? wordsResult.getJSONObject("车辆识别代号").getString("words") : null);
        license.setEngineNo(wordsResult.getJSONObject("发动机号码") != null ? wordsResult.getJSONObject("发动机号码").getString("words") : null);
        license.setLicenseModel(wordsResult.getJSONObject("品牌型号") != null ? wordsResult.getJSONObject("品牌型号").getString("words") : null);
        license.setType(wordsResult.getJSONObject("车辆类型") != null ? wordsResult.getJSONObject("车辆类型").getString("words") : null);
        license.setUseCharacter(wordsResult.getJSONObject("使用性质") != null ? wordsResult.getJSONObject("使用性质").getString("words") : null);
        license.setOwner(wordsResult.getJSONObject("所有人") != null ? wordsResult.getJSONObject("所有人").getString("words") : null);
        license.setAddress(wordsResult.getJSONObject("住址") != null ? wordsResult.getJSONObject("住址").getString("words") : null);
        license.setApprovedPerson(wordsResult.getJSONObject("核定载人数") != null ? wordsResult.getJSONObject("核定载人数").getString("words") : null);
        license.setWeight(wordsResult.getJSONObject("总质量") != null ? wordsResult.getJSONObject("总质量").getString("words") : null);
        license.setOutlineSize(wordsResult.getJSONObject("外廓尺寸") != null ? wordsResult.getJSONObject("外廓尺寸").getString("words") : null);
        license.setIssueDate(wordsResult.getJSONObject("发证日期") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("发证日期").getString("words")) : null);
        license.setRegisterDate(wordsResult.getJSONObject("注册日期") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("注册日期").getString("words")) : null);

        return license;
    }

    public IdentityCardVo identityCardDetect(byte[] imageBytes, boolean isMain) throws Exception {
        String accessToken = getAccessToken();

        String base64 = Base64.getEncoder().encodeToString(imageBytes);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> parameters = new LinkedMultiValueMap<>();
        parameters.add("image", base64);
        parameters.add("detect_direction", "true");
        parameters.add("id_card_side", isMain ? "front" : "back");
        parameters.add("detect_risk", "true");
        parameters.add("detect_photo", "false");
        parameters.add("detect_rectify", "true");

//        System.out.println(base64);
        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<MultiValueMap> requestEntity = new HttpEntity<>(parameters, headers);
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(String.format(URL_BAIDU_IDCARD, accessToken), requestEntity, String.class);

//        {"log_id": 3374241317740710780, "words_result_num": 1, "words_result": [{"words": "m AL"}], "language": 0}
        String body = responseEntity.getBody();
        System.out.println(body);
        JSONObject wordsResult = JSON.parseObject(body).getJSONObject("words_result");

        IdentityCardVo idCard = new IdentityCardVo();

        idCard.setName(wordsResult.getJSONObject("姓名") != null ? wordsResult.getJSONObject("姓名").getString("words") : null);
        idCard.setNation(wordsResult.getJSONObject("民族") != null ? wordsResult.getJSONObject("民族").getString("words") : null);
        idCard.setOriginAddress(wordsResult.getJSONObject("住址") != null ? wordsResult.getJSONObject("住址").getString("words") : null);
        idCard.setIdentityNumber(wordsResult.getJSONObject("公民身份号码") != null ? wordsResult.getJSONObject("公民身份号码").getString("words") : null);
        idCard.setBirthday(wordsResult.getJSONObject("出生") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("出生").getString("words")) : null);
        idCard.setGender(wordsResult.getJSONObject("性别") != null ? wordsResult.getJSONObject("性别").getString("words") : null);

        idCard.setIssueOrganization(wordsResult.getJSONObject("签发机关") != null ? wordsResult.getJSONObject("签发机关").getString("words") : null);
        idCard.setValidPeriodStart(wordsResult.getJSONObject("签发日期") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("签发日期").getString("words")) : null);
        idCard.setValidPeriodEnd(wordsResult.getJSONObject("失效日期") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("失效日期").getString("words")) : null);

        return idCard;
    }

    public DrivingLicenseVo drivingLicenseDetect(byte[] imageBytes) throws Exception {
        String accessToken = getAccessToken();

        String base64 = Base64.getEncoder().encodeToString(imageBytes);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> parameters = new LinkedMultiValueMap<>();
        parameters.add("image", base64);
        parameters.add("detect_direction", "true");
        parameters.add("unified_valid_period", "true");

//        System.out.println(base64);
        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<MultiValueMap> requestEntity = new HttpEntity<>(parameters, headers);
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(String.format(URL_BAIDU_DRIVER_LICENSE, accessToken), requestEntity, String.class);

//        {"log_id": 3374241317740710780, "words_result_num": 1, "words_result": [{"words": "m AL"}], "language": 0}
        String body = responseEntity.getBody();
        System.out.println(body);
        JSONObject wordsResult = JSON.parseObject(body).getJSONObject("words_result");

        DrivingLicenseVo license = new DrivingLicenseVo();

        license.setName(wordsResult.getJSONObject("姓名") != null ? wordsResult.getJSONObject("姓名").getString("words") : null);
        license.setCredentialNo(wordsResult.getJSONObject("证号") != null ? wordsResult.getJSONObject("证号").getString("words") : null);
        license.setAddress(wordsResult.getJSONObject("住址") != null ? wordsResult.getJSONObject("住址").getString("words") : null);
        license.setCountry(wordsResult.getJSONObject("国籍") != null ? wordsResult.getJSONObject("国籍").getString("words") : null);
        license.setDrivingClass(wordsResult.getJSONObject("准驾车型") != null ? wordsResult.getJSONObject("准驾车型").getString("words") : null);

        license.setFirstIssue(wordsResult.getJSONObject("初次领证日期") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("初次领证日期").getString("words")) : null);
        license.setValidPeriodStart(wordsResult.getJSONObject("有效起始日期") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("有效起始日期").getString("words")) : null);
        license.setValidPeriodEnd(wordsResult.getJSONObject("失效日期") != null ? DateUtil.parseBrithday(wordsResult.getJSONObject("失效日期").getString("words")) : null);

        return license;
    }


    public String textDetect(File image) throws Exception {
        String accessToken = getAccessToken();

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String, String> parameters = new LinkedMultiValueMap<>();

        FileInputStream fis = new FileInputStream(image);
        byte[] bytes = readNBytes(fis);
//        byte[] bytes = fis.readAllBytes();
//        System.out.println(bytes.length);

        String base64 = Base64.getEncoder().encodeToString(bytes);
        parameters.add("image", base64);
        parameters.add("language_type", "ENG");
        parameters.add("detect_language", "true");

//        System.out.println(base64);
        RestTemplate restTemplate = new RestTemplate();
        HttpEntity<MultiValueMap<String, String>> requestEntity = new HttpEntity<>(parameters, headers);
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(String.format(URL_BAIDU_TEXT_DETECT, accessToken), requestEntity, String.class);

//        {"log_id": 3374241317740710780, "words_result_num": 1, "words_result": [{"words": "m AL"}], "language": 0}
        String body = responseEntity.getBody();
//        System.out.println(body);
        JSONArray wordsResult = JSON.parseObject(body).getJSONArray("words_result");
        StringBuffer word = new StringBuffer();
        wordsResult.forEach(item -> {
            String text = JSON.parseObject(item.toString()).getString("words");
            if (StringUtils.isEmpty(text)) {
                return;
            }
            word.append(text);
        });
        return word.toString();
    }

    private String getAccessToken() {
        RestTemplate restTemplate = new RestTemplate();
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(String.format(URL_BAIDU_TOKEN, APP_KEY, SECRET_KEY), null, String.class);
        String body = responseEntity.getBody();
        return JSON.parseObject(body).getString("access_token");
    }

    private static final int DEFAULT_BUFFER_SIZE = 8192;
    private static final int MAX_BUFFER_SIZE = Integer.MAX_VALUE - 8;

    public static byte[] readNBytes(InputStream fis) throws IOException {
        int len = Integer.MAX_VALUE;
        if (len < 0) {
            throw new IllegalArgumentException("len < 0");
        }

        List<byte[]> bufs = null;
        byte[] result = null;
        int total = 0;
        int remaining = len;
        int n;
        do {
            byte[] buf = new byte[Math.min(remaining, DEFAULT_BUFFER_SIZE)];
            int nread = 0;

            // read to EOF which may read more or less than buffer size
            while ((n = fis.read(buf, nread,
                    Math.min(buf.length - nread, remaining))) > 0) {
                nread += n;
                remaining -= n;
            }

            if (nread > 0) {
                if (MAX_BUFFER_SIZE - total < nread) {
                    throw new OutOfMemoryError("Required array size too large");
                }
                total += nread;
                if (result == null) {
                    result = buf;
                } else {
                    if (bufs == null) {
                        bufs = new ArrayList<>();
                        bufs.add(result);
                    }
                    bufs.add(buf);
                }
            }
            // if the last call to read returned -1 or the number of bytes
            // requested have been read then break
        } while (n >= 0 && remaining > 0);

        if (bufs == null) {
            if (result == null) {
                return new byte[0];
            }
            return result.length == total ?
                    result : Arrays.copyOf(result, total);
        }

        result = new byte[total];
        int offset = 0;
        remaining = total;
        for (byte[] b : bufs) {
            int count = Math.min(b.length, remaining);
            System.arraycopy(b, 0, result, offset, count);
            offset += count;
            remaining -= count;
        }

        return result;
    }

    public static byte[] getBytesByUrl(String url) throws Exception {
        URL u = new URL(url);
        HttpURLConnection conn = (HttpURLConnection) u.openConnection();

        InputStream is = conn.getInputStream();
        BufferedInputStream bis = new BufferedInputStream(is);

        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        //缓存2KB
        final int BUFFER_SIZE = 4 * 1024;
        final int EOF = -1;

        int c;
        byte[] buf = new byte[BUFFER_SIZE];

        while (true) {
            c = bis.read(buf);
            if (c == EOF)
                break;
            baos.write(buf, 0, c);
        }

        conn.disconnect();
        is.close();

        byte[] data = baos.toByteArray();
        baos.flush();

        return data;
    }
}
