package com.jair.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jair.entity.common.AppInfo;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.HmacAlgorithms;
import org.apache.commons.codec.digest.HmacUtils;
import org.apache.tomcat.util.codec.binary.Base64;
import org.apache.tomcat.util.security.ConcurrentMessageDigest;
import org.apache.tomcat.util.security.MD5Encoder;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
public class UploadUtil {
    public static AppInfo appInfo;

    private static String thirdId = "1539894";
    private static String appKey = "jdGcS9oq0fEatFIX";
    private static String appSecret = "c3ae6bbbeb60cc84b802e09bf597bbe1d750db3e";
    private static String HTTP_HOST = "https://api.sskuaixiu.com";

//    private static String thirdId = "8907926";
//    private static String appKey = "oTXosH5OVKYWtqBS";
//    private static String appSecret = "2b35c294cdcdeafa979baf59756ae41026693a7e";
//    private static String HTTP_HOST = "https://apitest.sskuaixiu.com";
    private static String TOKEN_URL = HTTP_HOST + "/openapi/token/get";
    private static String UPLOAD_URL = HTTP_HOST + "/openapi/b2b-biz/file/upload";


    public void getAppInfo() throws Exception {
        if (checkToken()) {
            return;
        }
//        System.out.println("getAppInfo");
        RestTemplate restTemplate = new RestTemplate();
        String timestamp = String.valueOf(new Date().getTime());
        String encryptedSecret = MD5Encoder.encode(ConcurrentMessageDigest.digestMD5((appSecret + timestamp).getBytes()));

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("appKey", appKey);
        jsonObject.put("encryptedSecret", encryptedSecret);

        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        httpHeaders.add("third-id", thirdId);
        httpHeaders.add("timestamp", timestamp);

        HttpEntity<String> entity = new HttpEntity<>(jsonObject.toJSONString(), httpHeaders);
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(TOKEN_URL, entity, String.class);

        AppInfo token = JSON.parseObject(JSON.parseObject(responseEntity.getBody()).getString("data"), AppInfo.class);
        if (token == null || StringUtils.isEmpty(token.getToken())) {
            throw new Exception("获取token失败");
        }
        System.out.println("appInfo:" + JSON.toJSONString(token));
        appInfo = token;
    }

    public String upload(MultipartFile multipartFile) throws Exception {
        this.getAppInfo();

        RestTemplate restTemplate = new RestTemplate();
        String timestamp = String.valueOf(new Date().getTime());

        FileReq fileReq = new FileReq();
        fileReq.setFileName(multipartFile.getOriginalFilename());
        fileReq.setBytes(getFileBytes(multipartFile));

        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.setSerializationInclusion(JsonInclude.Include.NON_NULL);
        String params = objectMapper.writeValueAsString(fileReq);

        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        httpHeaders.add("third-id", thirdId);
        httpHeaders.add("timestamp", timestamp);
        httpHeaders.add("token", appInfo.getToken());
        httpHeaders.add("sign", generatorSign(params, appInfo.getToken(), timestamp));

        HttpEntity<String> entity = new HttpEntity<>(params, httpHeaders);
        ResponseEntity<String> responseEntity = restTemplate.postForEntity(UPLOAD_URL, entity, String.class);

        return JSON.parseObject(responseEntity.getBody()).getString("data");
    }

    private boolean checkToken() {
        if (appInfo == null) {
            return false;
        }

        Integer duration = appInfo.getActiveDuration() - 60 * 60;
        Date current = new Date();
        Date expiredDate = DateUtil.getAddTime(appInfo.getGenerateTime(), Calendar.SECOND, duration);
        if (current.getTime() >= expiredDate.getTime()) {
            return false;
        }

        return true;
    }

    public byte[] getFileBytes(MultipartFile multipartFile) throws Exception {
        return readAllBytes(multipartFile.getInputStream());
    }

    public byte[] getFileBytes(String filename) throws Exception {
        return readAllBytes(new FileInputStream(new File(filename)));
    }

    public static String generatorSign(String body, String token, String timestamp) {
        // ObjectMapper objMapper = new ObjectMapper();
        // objMapper.configure(JsonParser.Feature.ALLOW_UNQUOTED_CONTROL_CHARS, true);
        // Map<String, Object> dataMap = objMapper.convertValue(body, Map.class);
        Map<String, Object> dataMap = JSON.parseObject(body, Map.class);
        List<Map.Entry<String, Object>> dataList = dataMap.entrySet().stream()
                .filter(entry -> StringUtils.isNotEmpty(entry.getKey()) && null != entry.getValue())
                .collect(Collectors.toList());
        dataList.sort(Comparator.comparing(Map.Entry::getKey));
        String dataListStr = JSONObject.toJSONString(dataList);
        if (dataListStr.length() > 1024) {
            dataListStr = dataListStr.substring(0, 512) + "......" + dataListStr.substring(dataListStr.length() - 512);
        }
        log.info("签名数据:dataList={}", dataListStr);
        StringBuilder sb = new StringBuilder();
        dataList.stream().filter(item -> item.getValue() != null && StringUtils.isNotEmpty(item.getValue().toString()))
                .forEach(item -> sb.append(item.getKey()).append("=").append(item.getValue()).append("&"));
        sb.append("secret").append("=").append(appSecret).append("&");
        sb.append("token").append("=").append(token).append(timestamp);
        String signStr = sb.toString();
        if (signStr.length() > 2 * 1024) {
            signStr = signStr.substring(0, 1024) + "......" + signStr.substring(signStr.length() - 1024);
        }
        log.info("待加密字符串为：{}", signStr);
        HmacUtils hmacUtils = new HmacUtils(HmacAlgorithms.HMAC_SHA_256, appSecret.getBytes());
        return Base64.encodeBase64String(hmacUtils.hmac(sb.toString()));
    }

    public static byte[] readAllBytes(InputStream fis) throws IOException {
        int len = 2147483647;
        if (len < 0) {
            throw new IllegalArgumentException("len < 0");
        } else {
            List<byte[]> bufs = null;
            byte[] result = null;
            int total = 0;
            int remaining = len;

            int n;
            do {
                byte[] buf = new byte[Math.min(remaining, 8192)];

                int nread;
                for (nread = 0; (n = fis.read(buf, nread, Math.min(buf.length - nread, remaining))) > 0; remaining -= n) {
                    nread += n;
                }

                if (nread > 0) {
                    if (2147483639 - total < nread) {
                        throw new OutOfMemoryError("Required array size too large");
                    }

                    total += nread;
                    if (result == null) {
                        result = buf;
                    } else {
                        if (bufs == null) {
                            bufs = new ArrayList();
                            bufs.add(result);
                        }

                        bufs.add(buf);
                    }
                }
            } while (n >= 0 && remaining > 0);

            if (bufs == null) {
                if (result == null) {
                    return new byte[0];
                } else {
                    return result.length == total ? result : Arrays.copyOf(result, total);
                }
            } else {
                result = new byte[total];
                int offset = 0;
                remaining = total;

                int count;
                for (Iterator var12 = bufs.iterator(); var12.hasNext(); remaining -= count) {
                    byte[] b = (byte[]) var12.next();
                    count = Math.min(b.length, remaining);
                    System.arraycopy(b, 0, result, offset, count);
                    offset += count;
                }

                return result;
            }
        }
    }

    @Data
    class FileReq {
        private String fileName;
        private byte[] bytes;
    }
}
