package com.jair;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jair.entity.bo.Contract;
import com.jair.entity.common.TokenInfo;
import com.jair.entity.po.*;
import com.jair.mapper.ContractMapper;
import com.jair.mapper.FeePlanningMapper;
import com.jair.service.IContractService;
import com.jair.service.impl.PlanningServiceImpl;
import com.jair.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.text.ParseException;
import java.util.Arrays;
import java.util.Calendar;

@Slf4j
@RunWith(SpringRunner.class)
@SpringBootTest
public class ContractTest {

    @Autowired
    private IContractService sourceContractService;
    @Autowired
    ContractMapper contractMapper;
    @Autowired
    FeePlanningMapper planningMapper;

    @Test
    public void createCode() {
        for (int i = 0; i < 100; i++) {
            int type = i % 2;
            if (type == 0) {
                type = 2;
            }
            System.out.println(sourceContractService.genContractCode(type));
        }
    }

    @Test
    public void queryContract() {
        Page<Contract> page = new Page<>(2, 1);
        SearchContractPo se = new SearchContractPo();
        se.setType(2);
        System.out.println(contractMapper.queryDriverContracts(page, se));
    }

    @Test
    public void sourceSignContracts() throws Exception {
        SignContractPo signSourceContractPo = new SignContractPo();
        signSourceContractPo.setVehicleId(2);
        signSourceContractPo.setSourceId(1);
        signSourceContractPo.setStartDate(DateUtil.getAddDayTime(0));
        signSourceContractPo.setEndDate(DateUtil.getAddDayTime(180));
        signSourceContractPo.setOdograph(10000);
        signSourceContractPo.setOdographEv(200);
        signSourceContractPo.setRentAmount(3700);
        System.out.println(sourceContractService.signContract(getTokenInfo(), signSourceContractPo));
    }

    @Test
    public void saveRentPlanning() throws ParseException {
        Contract contract = new Contract();
        contract.setId(1);
        contract.setStartDate(DateUtil.parseDate("2020-01-01"));
        contract.setEndDate(DateUtil.getAddTime(DateUtil.parseDate("2020-01-01"), Calendar.DATE, 90));
        contract.setLeaseType(1);
        contract.setRentAmount(3000);

    }

    @Test
    public void updateContracts() {
        UpdateContractPo updateContractPo = new UpdateContractPo();
        updateContractPo.setContractId(2);
        updateContractPo.setStartDate(DateUtil.getAddDayTime(30));
        updateContractPo.setEndDate(DateUtil.getAddDayTime(80));
        updateContractPo.setOdograph(10000);
        updateContractPo.setOdographEv(0);
        updateContractPo.setDepositStatus(2);
        updateContractPo.setReturnDepositAmount(500);
        updateContractPo.setRentAmount(9000);
        System.out.println(sourceContractService.updateContract(getTokenInfo(), updateContractPo));
    }

    @Test
    public void returnVehicle() {
        ReturnVehiclePo returnVehiclePo = new ReturnVehiclePo();
        returnVehiclePo.setContractId(2);
        returnVehiclePo.setAccident(1);
        returnVehiclePo.setAccidentFiles(Arrays.asList(111, 222));
        returnVehiclePo.setLicense(0);
        returnVehiclePo.setBreakRulesAmount(500);
        returnVehiclePo.setBreakRulesDeduction(3);

        System.out.println(sourceContractService.returnVehicle(getTokenInfo(), returnVehiclePo));
    }

    @Test
    public void sourceReturnDeposit() {
        ReturnDepositPo returnDepositPo = new ReturnDepositPo();
        returnDepositPo.setContractId(1);
        returnDepositPo.setReturnAmount(1000);
        returnDepositPo.setRemark("退1000");
        System.out.println(sourceContractService.returnDeposit(getTokenInfo(), returnDepositPo));
    }

    @Test
    public void sourceTerminateContracts() {
        TerminateContractPo terminatePo = new TerminateContractPo();
        terminatePo.setContractId(1);
        System.out.println(sourceContractService.terminateContract(getTokenInfo(), terminatePo));
    }


    @Test
    public void list() {
        SearchPlanningPo searchPlanningPo = new SearchPlanningPo();
    }

    public TokenInfo getTokenInfo() {
        TokenInfo tokenInfo = new TokenInfo();
        tokenInfo.setId(2);
        tokenInfo.setRealName("张三");
        tokenInfo.setCompanyId(1);
        return tokenInfo;
    }
}
